package com.bsz.school_send_back_end.controller;


import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bsz.school_send_back_end.annotation.AuthCheck;
import com.bsz.school_send_back_end.common.BaseResponse;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.common.ResultUtils;
import com.bsz.school_send_back_end.contant.CommonConstant;
import com.bsz.school_send_back_end.contant.UserConstant;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.*;
import com.bsz.school_send_back_end.model.dto.CommonRequest;
import com.bsz.school_send_back_end.model.dto.order.*;
import com.bsz.school_send_back_end.model.vo.OrdersExcelVO;
import com.bsz.school_send_back_end.model.vo.OrdersVO;
import com.bsz.school_send_back_end.service.*;
import com.bsz.school_send_back_end.utils.ExcelUtils;
import com.bsz.school_send_back_end.utils.SqlUtils;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;


/**
 * 订单相关接口
 **/
@Slf4j
@RestController
@Tag(name = "订单接口")
@RequestMapping("/orders")
public class OrdersController {

    @Resource
    private UserService userService;

    @Resource
    private DishesService dishesService;

    @Resource
    private OrdersService ordersService;

    @Resource
    private BusinessService businessService;

    @Resource
    private OrderDetailsService orderDetailsService;

    /**
     * 创建订单
     *
     * @return 订单id
     */
    @PostMapping("/add")
    public BaseResponse<Long> addOrders(@RequestBody OrderAddRequest orderAddRequest, HttpServletRequest request) {
        if (orderAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 提取订单
        Orders orders = new Orders();
        User loginUser = userService.getLoginUser(request);
        BeanUtils.copyProperties(orderAddRequest, orders);
        orders.setUserId(loginUser.getId());
        ordersService.validOrder(orders);
        // 提取订单详情
        List<OrderDetailAddRequest> detailAddRequest = orderAddRequest.getOrderDetailAddRequest();
        List<OrderDetails> orderDetailsList = detailAddRequest.stream().map(item -> {
            OrderDetails orderDetails = new OrderDetails();
            BeanUtils.copyProperties(item, orderDetails);
            orderDetailsService.validOrderDetail(orderDetails);
            Long dishesId = orderDetails.getDishesId();
            // 根据菜品设置价格
            Dishes dishes = dishesService.getById(dishesId);
            BigDecimal dishesPrice = BigDecimal.valueOf(dishes.getDishesPrice());
            orderDetails.setPrice(dishesPrice);
            double subtotal = dishes.getDishesPrice() * orderDetails.getQuantity();
            orderDetails.setSubtotal(BigDecimal.valueOf(subtotal));
            return orderDetails;
        }).toList();
        // 创建订单
        long orderId = ordersService.addOrder(orders, orderDetailsList);
        return ResultUtils.success(orderId, "订单创建成功");
    }

    /**
     * 取消订单
     */
    @PostMapping("/cancel")
    public BaseResponse<Boolean> cancelOrder(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        Long orderId = commonRequest.getId();
        if (orderId == null || orderId <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 校验订单
        Orders orders = ordersService.getById(orderId);
        ThrowUtils.throwIf(orders == null, ErrorCode.NOT_FOUND_ERROR);
        ThrowUtils.throwIf(orders.getState() != 0, ErrorCode.OPERATION_ERROR, "订单状态错误");
        // 判断是否为自己的订单
        User loginUser = userService.getLoginUser(request);
        if (!orders.getUserId().equals(loginUser.getId())) {
            throw new BusinessException(ErrorCode.NO_AUTH);
        }
        // 修改订单状态
        orders.setState(3);
        boolean update = ordersService.updateById(orders);
        ThrowUtils.throwIf(!update, ErrorCode.SYSTEM_ERROR);
        return ResultUtils.success(true);
    }

    /**
     * 根据 id 获取订单信息（仅管理员）
     */
    @GetMapping("/get")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<OrdersVO> getOrderVOById(@RequestParam Long id) {
        if (id <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Orders orders = ordersService.getById(id);
        ThrowUtils.throwIf(orders == null, ErrorCode.NOT_FOUND_ERROR);
        return ResultUtils.success(ordersService.getOrdersVO(orders));
    }

    /**
     * 根据 id 获取订单信息
     */
    @GetMapping("/get/my")
    @AuthCheck(mustRole = UserConstant.DEFAULT_ROLE)
    public BaseResponse<OrdersVO> getOrderVOByMyId(@RequestParam Long id, HttpServletRequest request) {
        if (id <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        User loginUser = userService.getLoginUser(request);
        Orders orders = ordersService.getOne(Wrappers.<Orders>lambdaQuery().eq(Orders::getId, id)
                .eq(Orders::getUserId, loginUser.getId()));
        ThrowUtils.throwIf(orders == null, ErrorCode.NOT_FOUND_ERROR);
        return ResultUtils.success(ordersService.getOrdersVO(orders));
    }

    /**
     * 分页获取订单列表（管理员）
     */
    @PostMapping("/list/page")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Page<Orders>> listOrdersByPage(@RequestBody OrderQueryRequest orderQueryRequest) {
        long current = orderQueryRequest.getCurrent();
        long size = orderQueryRequest.getPageSize();
        Page<Orders> ordersPage = ordersService.page(new Page<>(current, size),
                ordersService.getQueryWrapper(orderQueryRequest));
        return ResultUtils.success(ordersPage);
    }

    /**
     * 获取我的订单
     */
    @PostMapping("/my/page")
    public BaseResponse<Page<OrdersVO>> getMyOrders(@RequestBody MyOrderQueryRequest myOrderQueryRequest, HttpServletRequest request) {
        if (myOrderQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        long current = myOrderQueryRequest.getCurrent();
        long size = myOrderQueryRequest.getPageSize();
        User loginUser = userService.getLoginUser(request);
        OrderQueryRequest orderQueryRequest = new OrderQueryRequest();
        BeanUtils.copyProperties(myOrderQueryRequest, orderQueryRequest);
        QueryWrapper<Orders> queryWrapper = ordersService.getQueryWrapper(orderQueryRequest);
        if (loginUser.getUserRole().equals(UserConstant.BUSINESS_ROLE)) {
            // 商家获取订单
            Business loginBusiness = businessService.getLoginBusiness(request);
            queryWrapper.eq("businessId", loginBusiness.getId());
        } else {
            // 用户获取订单
            queryWrapper.eq("userId", loginUser.getId());
        }
        Page<Orders> ordersPage = ordersService.page(new Page<>(current, size),
                queryWrapper);
        Page<OrdersVO> ordersVOPage = new Page<>(current, size, ordersPage.getTotal());
        List<OrdersVO> ordersVOList = ordersService.getOrdersVO(ordersPage.getRecords());
        ordersVOPage.setRecords(ordersVOList);
        return ResultUtils.success(ordersVOPage);
    }

    /**
     * 订单统计
     */
    @PostMapping("/count")
    @AuthCheck(mustRole = UserConstant.BUSINESS_ROLE)
    public BaseResponse<String> ordersCount(@RequestBody OrderCountRequest orderCountRequest) {
        String type = orderCountRequest.getType();
        Integer state = orderCountRequest.getState();
        Long businessId = orderCountRequest.getBusinessId();
        String startTime = orderCountRequest.getStartTime();
        String endTime = orderCountRequest.getEndTime();
        ThrowUtils.throwIf(StringUtils.isBlank(type), ErrorCode.PARAMS_ERROR);
        QueryWrapper<Orders> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(state != null, "state", state);
        queryWrapper.eq(businessId != null, "businessId", businessId);
        return getStringBaseResponse(type, startTime, endTime, queryWrapper);
    }

    /**
     * 订单金额统计
     */
    @PostMapping("/count/money")
    @AuthCheck(mustRole = UserConstant.BUSINESS_ROLE)
    public BaseResponse<List<BigDecimal>> ordersCountMoney(@RequestBody OrderCountRequest orderCountRequest) {
        // 获取查询类型
        String type = orderCountRequest.getType();
        ThrowUtils.throwIf(StringUtils.isBlank(type), ErrorCode.PARAMS_ERROR);
        OrderQueryRequest orderQueryRequest = new OrderQueryRequest();
        BeanUtils.copyProperties(orderCountRequest, orderQueryRequest);
        List<BigDecimal> moneyCountList = new ArrayList<>();
        if (type.equals("week")) {
            LocalDate today = LocalDate.now();
            DateTimeFormatter startFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd 00:00:00");
            DateTimeFormatter endFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd 23:59:59");
            // 遍历查询7天数据，放入数组
            for (int i = 0; i <= 6; i++) {
                LocalDate date = today.minusDays(i);
                orderQueryRequest.setStartTime(date.format(startFormatter));
                orderQueryRequest.setEndTime(date.format(endFormatter));
                QueryWrapper<Orders> queryWrapper = ordersService.getQueryWrapper(orderQueryRequest);
                List<Orders> ordertList = ordersService.list(queryWrapper);
                BigDecimal money = new BigDecimal("0");
                for (Orders order : ordertList) {
                    money = money.add(order.getTotalPrice());
                }
                moneyCountList.add(money);
            }
        } else {
            QueryWrapper<Orders> queryWrapper = ordersService.getQueryWrapper(orderQueryRequest);
            List<Orders> ordertList = ordersService.list(queryWrapper);
            BigDecimal money = new BigDecimal("0");
            for (Orders order : ordertList) {
                money = money.add(order.getTotalPrice());
            }
            moneyCountList.add(money);
        }
        return ResultUtils.success(moneyCountList);
    }

    /**
     * 订单数量统计
     */
    @PostMapping("/count/number")
    @AuthCheck(mustRole = UserConstant.BUSINESS_ROLE)
    public BaseResponse<List<Long>> ordersCountNumber(@RequestBody OrderCountRequest orderCountRequest) {
        // 获取查询类型
        String type = orderCountRequest.getType();
        ThrowUtils.throwIf(StringUtils.isBlank(type), ErrorCode.PARAMS_ERROR);
        OrderQueryRequest orderQueryRequest = new OrderQueryRequest();
        BeanUtils.copyProperties(orderCountRequest, orderQueryRequest);
        List<Long> numberCountList = new ArrayList<>();
        if (type.equals("week")) {
            LocalDate today = LocalDate.now();
            DateTimeFormatter startFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd 00:00:00");
            DateTimeFormatter endFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd 23:59:59");
            // 遍历查询7天数据，放入数组
            for (int i = 0; i <= 6; i++) {
                LocalDate date = today.minusDays(i);
                orderQueryRequest.setStartTime(date.format(startFormatter));
                orderQueryRequest.setEndTime(date.format(endFormatter));
                QueryWrapper<Orders> queryWrapper = ordersService.getQueryWrapper(orderQueryRequest);
                long count = ordersService.count(queryWrapper );
                numberCountList.add(count);
            }
        } else {
            QueryWrapper<Orders> queryWrapper = ordersService.getQueryWrapper(orderQueryRequest);
            long count = ordersService.count(queryWrapper);
            numberCountList.add(count);
        }
        return ResultUtils.success(numberCountList);
    }

    /**
     * 订单统计(web端)
     */
    @PostMapping("/count/web")
    @AuthCheck(mustRole = UserConstant.BUSINESS_ROLE)
    public BaseResponse<String> ordersCountByWeb(@RequestBody OrderCountRequest orderCountRequest) {
        String type = orderCountRequest.getType();
        Integer state = orderCountRequest.getState();
        String businessName = orderCountRequest.getBusinessName();
        Long businessState = orderCountRequest.getBusinessState();
        String startTime = orderCountRequest.getStartTime();
        String endTime = orderCountRequest.getEndTime();
        Long categoryId = orderCountRequest.getCategoryId();
        List<Long> businessIdList = new ArrayList<>();
        ThrowUtils.throwIf(StringUtils.isBlank(type), ErrorCode.PARAMS_ERROR);
        if (categoryId != null || businessName != null || businessState != null) {
            QueryWrapper<Business> wrapper = new QueryWrapper<>();
            wrapper.eq(StringUtils.isNotBlank(businessName), "businessName", businessName);
            wrapper.eq(businessState != null, "state", businessState);
            wrapper.eq(categoryId != null, "categoryId", categoryId);
            List<Business> businessList = businessService.list(wrapper);
            if (CollectionUtils.isEmpty(businessList)) {
                return ResultUtils.success("0");
            }
            businessIdList = businessList.stream().map(Business::getId).collect(Collectors.toList());
        }
        QueryWrapper<Orders> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(state != null, "state", state);
        queryWrapper.in(!CollectionUtils.isEmpty(businessIdList), "businessId", businessIdList);
        return getStringBaseResponse(type, startTime, endTime, queryWrapper);
    }

    @PostMapping("/download")
    public void download(@RequestBody OrderQueryRequest orderQueryRequest, HttpServletResponse response) throws IOException {
        String startTime = orderQueryRequest.getStartTime();
        String endTime = orderQueryRequest.getEndTime();
        Long businessId = orderQueryRequest.getBusinessId();
        String sortOrder = orderQueryRequest.getSortOrder();
        String sortField = orderQueryRequest.getSortField();
        // 获取数据
        QueryWrapper<Orders> wrapper = new QueryWrapper<>();
        wrapper.ge(StringUtils.isNotBlank(startTime), "createTime", startTime);
        wrapper.le(StringUtils.isNotBlank(endTime), "createTime", endTime);
        wrapper.eq("businessId", businessId).and(item -> item.eq("state", 1).or().eq("state", 2));
        wrapper.orderBy(SqlUtils.validSortField(sortField), sortOrder.equals(CommonConstant.SORT_ORDER_ASC),
                sortField);
        List<Orders> ordersList = ordersService.list(wrapper);
        List<OrdersExcelVO> ordersExcelVOList = ordersList.stream().map(item -> {
            OrdersExcelVO ordersExcelVO = new OrdersExcelVO();
            BeanUtils.copyProperties(item, ordersExcelVO);
            //改变订单支付状态
            if (item.getState() == 1) {
                ordersExcelVO.setState("已完成");
            } else if (item.getState() == 2) {
                ordersExcelVO.setState("已退款");
            }
            // Date转字符串
            ordersExcelVO.setCreateTime(ExcelUtils.dateToString(item.getCreateTime()));
            return ordersExcelVO;
        }).collect(Collectors.toList());
        // 设置导出名称
        ExcelUtils.setExcelResponseProp(response, "订单信息");
        // 获取输出流名称
        OutputStream outputStream = response.getOutputStream();
        // 这里 需要指定写用哪个class去写，然后写到第一个sheet，名字为模板 然后文件流会自动关闭
        EasyExcel.write(outputStream, OrdersExcelVO.class)  // 对应实体类
                .sheet("订单数据")  // sheet页名称
                .doWrite(ordersExcelVOList); // 导出的数据集合
    }

    @PostMapping("/delete")
    @AuthCheck(mustRole = UserConstant.DEFAULT_ROLE)
    public BaseResponse<Boolean> deleteOrder (@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        if (commonRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long orderId = commonRequest.getId();
        LambdaQueryWrapper<Orders> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(Orders::getId, orderId);
        Orders orders = ordersService.getOne(wrapper);
        User loginUser = userService.getLoginUser(request);
        Long loginUserId = loginUser.getId();
        ThrowUtils.throwIf(!orders.getUserId().equals(loginUserId), ErrorCode.OPERATION_ERROR,
                "当前用户不是该订单所有者");
        log.info("oldOrders:" + orders);
        LambdaUpdateWrapper<Orders> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(Orders::getIsDelete,1);
        updateWrapper.eq(Orders::getId, orderId);
        boolean update = ordersService.update(updateWrapper);
        log.info("orders:" + orders);
        ThrowUtils.throwIf(!update, ErrorCode.SYSTEM_ERROR);

        return ResultUtils.success(true);
    }

    @PostMapping("/update")
    @AuthCheck(mustRole = UserConstant.BUSINESS_ROLE)
    public BaseResponse<Boolean> updateOrderState(@RequestBody CommonRequest commonRequest,HttpServletRequest request) {
        if (commonRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        LambdaQueryWrapper<Orders> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Orders::getId, id);
        Orders orders = ordersService.getOne(queryWrapper);
        Business loginBusiness = businessService.getLoginBusiness(request);
        Long businessId = loginBusiness.getId();
        ThrowUtils.throwIf(!businessId.equals(orders.getBusinessId()), ErrorCode.FORBIDDEN_ERROR,
                "当前订单不属于当前登录用户");
        ThrowUtils.throwIf(!orders.getState().equals(1), ErrorCode.OPERATION_ERROR,"当前订单状态不是已支付状态");
        LambdaUpdateWrapper<Orders> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(Orders::getState,4);
        updateWrapper.eq(Orders::getId, id);
        boolean update = ordersService.update(updateWrapper);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR,"修改订单状态失败");

        return ResultUtils.success(true);
    }

    @PostMapping("/update/user")
    @AuthCheck(mustRole = UserConstant.DEFAULT_ROLE)
    public BaseResponse<Boolean> updateOrderStateByUser (@RequestBody CommonRequest commonRequest, HttpServletRequest request){
        if (commonRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        Long orderId = commonRequest.getId();
        LambdaQueryWrapper<Orders> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Orders::getId, orderId);
        Orders orders = ordersService.getOne(queryWrapper);
        User loginUser = userService.getLoginUser(request);
        ThrowUtils.throwIf(!loginUser.getId().equals(orders.getUserId()), ErrorCode.OPERATION_ERROR,
                "当前订单不属于该用户");
        ThrowUtils.throwIf(!orders.getState().equals(4), ErrorCode.OPERATION_ERROR,"该订单未出餐");
        LambdaUpdateWrapper<Orders> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(Orders::getState, 5);
        updateWrapper.eq(Orders::getId, orderId);
        boolean update = ordersService.update(updateWrapper);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR,"修改订单状态失败");
        return ResultUtils.success(true);
    }


    private BaseResponse<String> getStringBaseResponse(String type, String startTime, String endTime, QueryWrapper<Orders> queryWrapper) {
        queryWrapper.ge(StringUtils.isNotBlank(startTime), "createTime", startTime);
        queryWrapper.le(StringUtils.isNotBlank(endTime), "createTime", endTime);
        if (type.equals("money")) {
            List<Orders> ordertList = ordersService.list(queryWrapper);
            BigDecimal money = new BigDecimal("0");
            for (Orders order : ordertList) {
                money = money.add(order.getTotalPrice());
            }
            String strMoney = String.valueOf(money);
            return ResultUtils.success(strMoney);
        } else if (type.equals("number")) {
            long count = ordersService.count(queryWrapper);
            String strNumber = String.valueOf(count);
            return ResultUtils.success(strNumber);
        } else {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "type不正确");
        }
    }
}


