package com.bsz.school_send_back_end.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.contant.CommonConstant;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.mapper.OrdersMapper;
import com.bsz.school_send_back_end.model.domain.Business;
import com.bsz.school_send_back_end.model.domain.OrderDetails;
import com.bsz.school_send_back_end.model.domain.Orders;
import com.bsz.school_send_back_end.model.dto.order.OrderQueryRequest;
import com.bsz.school_send_back_end.model.vo.BusinessVO;
import com.bsz.school_send_back_end.model.vo.OrderDetailsVO;
import com.bsz.school_send_back_end.model.vo.OrdersVO;
import com.bsz.school_send_back_end.service.BusinessService;
import com.bsz.school_send_back_end.service.OrderDetailsService;
import com.bsz.school_send_back_end.service.OrdersService;
import com.bsz.school_send_back_end.utils.RegexUtils;
import com.bsz.school_send_back_end.utils.SqlUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author 玄德
 * @description 针对表【orders(订单表)】的数据库操作Service实现
 * @createDate 2024-04-05 17:57:44
 */
@Service
public class OrdersServiceImpl extends ServiceImpl<OrdersMapper, Orders>
        implements OrdersService {

    @Resource
    private BusinessService businessService;

    @Resource
    private OrderDetailsService orderDetailsService;


    @Override
    @Transactional(rollbackFor = Exception.class)
    public synchronized long addOrder(Orders orders, List<OrderDetails> orderDetailsList) {
        boolean saveOrder = this.save(orders);
        if (!saveOrder) {
            throw new BusinessException(ErrorCode.SYSTEM_ERROR, "创建订单失败，数据库错误");
        }
        // 创建订单菜品
        for (OrderDetails orderDetails : orderDetailsList) {
            // 绑定订单和订单详情
            orderDetails.setOrderId(orders.getId());
            boolean saveDetail = orderDetailsService.save(orderDetails);
            if (!saveDetail) {
                log.error("创建订单菜品失败，订单id：" + orders.getId());
                throw new BusinessException(ErrorCode.OPERATION_ERROR, "订单菜品创建失败");
            }
        }
        return orders.getId();
    }

    @Override
    public void validOrder(Orders orders) {
        String userName = orders.getUserName();
        String phone = orders.getPhone();
        Long userId = orders.getUserId();
        Long businessId = orders.getBusinessId();
        BigDecimal totalPrice = orders.getTotalPrice();
        Integer pickupMethod = orders.getPickupMethod();
        Integer payMethod = orders.getPayMethod();
        if (StringUtils.isAnyBlank(userName, phone)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        if (StringUtils.isNotBlank(phone) && RegexUtils.isPhoneInvalid(phone)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "手机号格式错误");
        }
        if (ObjectUtils.isEmpty(totalPrice) || totalPrice.floatValue() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "价格错误");
        }
        if (ObjectUtils.anyNull(userId, businessId, payMethod, pickupMethod)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "参数不全");
        }
    }

    @Override
    public OrdersVO getOrdersVO(Orders orders) {
        if (orders == null) {
            return null;
        }
        OrdersVO ordersVO = new OrdersVO();
        BeanUtils.copyProperties(orders, ordersVO);
        // 关联查询商家信息
        Long businessId = orders.getBusinessId();
        Business business = null;
        if (businessId != null && businessId > 0) {
            business = businessService.getById(businessId);
        }
        BusinessVO businessVO = businessService.getBusinessVO(business);
        ordersVO.setBusinessVO(businessVO);
        // 关联查询订单详情信息
        Long orderId = orders.getId();
        QueryWrapper<OrderDetails> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("orderId", orderId);
        List<OrderDetails> orderDetailsList = orderDetailsService.list(queryWrapper);
        List<OrderDetailsVO> orderDetailsVOList = orderDetailsService.getOrderDetailsVO(orderDetailsList);
        ordersVO.setOrderDetailsVOList(orderDetailsVOList);
        // 关联查询评价状态

        // 设置商品数量
        Integer number = 0;
        for (OrderDetailsVO detailsVO : orderDetailsVOList) {
            number += detailsVO.getQuantity();
        }
        ordersVO.setNumber(number);
        return ordersVO;
    }

    @Override
    public List<OrdersVO> getOrdersVO(List<Orders> ordersList) {
        if (CollectionUtils.isEmpty(ordersList)) {
            return new ArrayList<>();
        }
        return ordersList.stream().map(this::getOrdersVO).collect(Collectors.toList());
    }

    @Override
    public QueryWrapper<Orders> getQueryWrapper(OrderQueryRequest orderQueryRequest) {
        if (orderQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "请求参数为空");
        }
        Long id = orderQueryRequest.getId();
        Long userId = orderQueryRequest.getUserId();
        Long businessId = orderQueryRequest.getBusinessId();
        String pickupCode = orderQueryRequest.getPickupCode();
        Integer pickupMethod = orderQueryRequest.getPickupMethod();
        Integer state = orderQueryRequest.getState();
        String startTime = orderQueryRequest.getStartTime();
        String endTime = orderQueryRequest.getEndTime();
        String sortField = orderQueryRequest.getSortField();
        String sortOrder = orderQueryRequest.getSortOrder();
        QueryWrapper<Orders> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(ObjectUtils.isNotEmpty(id), "id", id);
        queryWrapper.eq(ObjectUtils.isNotEmpty(userId), "userId", userId);
        queryWrapper.eq(ObjectUtils.isNotEmpty(businessId), "businessId", businessId);
        queryWrapper.eq(ObjectUtils.isNotEmpty(pickupMethod), "pickupMethod", pickupMethod);
        queryWrapper.eq(ObjectUtils.isNotEmpty(state), "state", state);
        queryWrapper.ge(StringUtils.isNotBlank(pickupCode), "pickupCode", pickupCode);
        queryWrapper.ge(StringUtils.isNotBlank(startTime), "createTime", startTime);
        queryWrapper.le(StringUtils.isNotBlank(endTime), "createTime", endTime);
        queryWrapper.orderBy(SqlUtils.validSortField(sortField), sortOrder.equals(CommonConstant.SORT_ORDER_ASC),
                sortField);
        return queryWrapper;
    }
}